/*
 Name:		STM32_USART_DMA.ino
 Created:	18.10.2021
 Author:	   Ulli
*/


#include <UrsUsart1Dma.h>

// Texte, die per USART im DMA-Betrieb gesendet werden.
// =============================
const char pangram1[] = "The quick brown fox jumps over the lazy dog\n";
const char pangram2[] = "Jeder wackere Bayer vertilgt bequem zwo Pfund Kalbshaxen\n";


// Elemente zum Betriebsmodus
// =============================
constexpr uint8_t modePinIn = PA8; // Betriebsmodus festlegen. Der RS232-To-USB-Dongle kann z.B. ber die RTS-Leitung diesen Pin ansprechen.
constexpr uint8_t modePinOut = PB15; // Betriebsmodus rckmelden. Der RS232-To-USB-Dongle kann z.B. ber die DTR-Leitung diesen Pin auslesen.

// Die Applikation kennt zwei Betriebsmodi
enum class SerialModes {
   Dma,   // schnelle Datenausgabe per DMA
   Serial // "normale" Serielle Schnittstelle (Echo-Funktion)
};
SerialModes currentMode = SerialModes::Dma;      // Der aktuelle Betriebsmodus
volatile SerialModes newMode = SerialModes::Dma; // Betriebsmodus in den gewechselt werden soll.

// Erkennt eine Vernderung am modePin.
// Der eigentliche Wechsel des Betriebsmodus erfolgt in loop() um Synchrnisationkonflikte zu vermeiden.
void modeChangeInterruptHandler() {
   if (digitalRead(modePinIn)) {
      newMode = SerialModes::Dma;
   }
   else {
      newMode = SerialModes::Serial;
   }
}

// Wechselt den Betriebsmodus
void setMode() {
   while (ursUsart1Dma.isBusy()) {} // Ende der bertragung abwarten
   while (Serial.available()) {     // Serial1 leer lesen
      Serial.read();
   }

   currentMode = newMode;
   if (currentMode == SerialModes::Dma) {
      ursUsart1Dma.begin(2000000); // Auf DMA-Betrieb umschalten
      Serial.println("Mode: DMA"); // Neuen Betriebsmodus zurck melden ...
      digitalWrite(modePinOut, HIGH); 

   }
   else {
      Serial1.begin(2000000);         // Auf "normalen" seriellen Betrieb umschalten
      Serial.println("Mode: Serial"); // Neuen Betriebsmodus zurck melden ...
      digitalWrite(modePinOut, LOW);
   }
}
// =========================


void setup() {
   pinMode(modePinIn, INPUT);
   pinMode(modePinOut, OUTPUT);

   delay(3000); // Der Treiber muss sich nach dem Programmieren wieder bei Windows anmelden

   Serial.begin(115200); // Standard USART (USB-USART) nur zur Protokollierung
   Serial.println("\nSTM32 USART-DMA Test");
   while (Serial.available()) { // evtl. Mll auslesen
      Serial.read();
   }

   attachInterrupt(modePinIn, modeChangeInterruptHandler, CHANGE); // modePinIn-PinChange-Interrupt 

   modeChangeInterruptHandler(); // modePinIn auslesen
   setMode(); // Initiale Einstellung des Betriebsmodus

   Serial.println("loop running");
}

void loop() {
   if (newMode != currentMode) { // Betriebsmodus gewechselt?
      setMode();                 // Betriebsmodus einstellen.
   }

   // Betriebsmodus Serial:
   // Empfangene Zeichen zurck senden (Echo)
   if (currentMode == SerialModes::Serial)
      if (Serial1.available())                // Nicht blockieren!
         Serial1.write((char)Serial1.read()); // Echo

   // Betriebsmodus DMA:
   // Beim Empfang eines Zeichens die Pangramme per USART mit DMA-Kontrolle versenden
   if (currentMode == SerialModes::Dma)
      if (Serial.available()) {
         while (Serial.available()) { // evtl. anstehede Zeichen lschen
            Serial.read();
         }

         Serial.println("Start xmit");

         ursUsart1Dma.xmit((const uint8_t*)pangram1, sizeof(pangram1) - 1);
         ursUsart1Dma.xmit((const uint8_t*)pangram2, sizeof(pangram2) - 1);

         Serial.println("Done");
      }
}
